import { CandidateEntity } from '@modules/ad/core/domain/candidate.entity';
import { Completer } from './completer.abstract';
import { Role } from '@modules/ad/core/domain/ad.types';
import {
  Waypoint as WaypointValueObject,
  WaypointProps,
} from '@modules/ad/core/domain/value-objects/waypoint.value-object';
import { Waypoint } from '../../../types/waypoint.type';
import { WayStepsCreator } from '@modules/ad/core/domain/waysteps-creator.service';

/**
 * Complete candidates by setting driver and crew waypoints
 */
export class PassengerOrientedWaypointsCompleter extends Completer {
  complete = async (
    candidates: CandidateEntity[],
  ): Promise<CandidateEntity[]> => {
    candidates.forEach((candidate: CandidateEntity) => {
      const carpoolPathCreator = new WayStepsCreator(
        candidate.getProps().role == Role.DRIVER
          ? candidate.getProps().waypoints.map(
              (waypoint: WaypointProps) =>
                new WaypointValueObject({
                  position: waypoint.position,
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            )
          : this.query.waypoints.map(
              (waypoint: Waypoint) =>
                new WaypointValueObject({
                  position: waypoint.position,
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            ),
        candidate.getProps().role == Role.PASSENGER
          ? candidate.getProps().waypoints.map(
              (waypoint: WaypointProps) =>
                new WaypointValueObject({
                  position: waypoint.position,
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            )
          : this.query.waypoints.map(
              (waypoint: Waypoint) =>
                new WaypointValueObject({
                  position: waypoint.position,
                  lon: waypoint.lon,
                  lat: waypoint.lat,
                }),
            ),
      );
      candidate.setWaySteps(carpoolPathCreator.getCrewCarpoolPath());
    });
    // console.log(
    //   candidates[0]
    //     .getProps()
    //     .waySteps?.map((waystep: WayStep) => waystep.actors),
    // );
    return candidates;
  };
}

// complete = async (candidates: Candidate[]): Promise<Candidate[]> => {
//     candidates.forEach( (candidate: Candidate) => {
//       if (candidate.role == Role.DRIVER) {
//         candidate.driverWaypoints = th
//       }

//     return candidates;
//   }
