import { Role } from './ad.types';
import { Target } from './candidate.types';
import { Actor } from './value-objects/actor.value-object';
import { Waypoint } from './value-objects/waypoint.value-object';
import { WayStep } from './value-objects/waystep.value-object';

export class WayStepsCreator {
  constructor(
    private readonly driverWaypoints: Waypoint[],
    private readonly passengerWaypoints: Waypoint[],
  ) {}

  public getCrewCarpoolPath = (): WayStep[] => this._createPassengerWaysteps();

  private _createPassengerWaysteps = (): WayStep[] => {
    const waysteps: WayStep[] = [];
    this.passengerWaypoints.forEach((passengerWaypoint: Waypoint) => {
      const waystep: WayStep = new WayStep({
        lon: passengerWaypoint.lon,
        lat: passengerWaypoint.lat,
        position: passengerWaypoint.position,
        actors: [
          new Actor({
            role: Role.PASSENGER,
            target: this._getTarget(
              passengerWaypoint.position,
              this.passengerWaypoints,
            ),
          }),
        ],
      });
      if (
        this.driverWaypoints.filter((driverWaypoint: Waypoint) =>
          this._isSameWaypoint(driverWaypoint, passengerWaypoint),
        ).length > 0
      ) {
        waystep.actors.push(
          new Actor({
            role: Role.DRIVER,
            target: Target.NEUTRAL,
          }),
        );
      }
      waysteps.push(waystep);
    });
    return waysteps;
  };

  private _isSameWaypoint = (
    waypoint1: Waypoint,
    waypoint2: Waypoint,
  ): boolean =>
    waypoint1.lon === waypoint2.lon && waypoint1.lat === waypoint2.lat;

  private _getTarget = (position: number, waypoints: Waypoint[]): Target =>
    position == 0
      ? Target.START
      : position ==
        Math.max(...waypoints.map((waypoint: Waypoint) => waypoint.position))
      ? Target.FINISH
      : Target.INTERMEDIATE;
}
