import { ArgumentOutOfRangeException } from '@mobicoop/ddd-library';
import { Role } from '@modules/ad/core/domain/ad.types';
import { Target } from '@modules/ad/core/domain/candidate.types';
import { Actor } from '@modules/ad/core/domain/value-objects/actor.value-object';
import { WayStep } from '@modules/ad/core/domain/value-objects/waystep.value-object';

describe('WayStep value object', () => {
  it('should create a waystep value object', () => {
    const wayStepVO = new WayStep({
      lat: 48.689445,
      lon: 6.17651,
      position: 0,
      actors: [
        new Actor({
          role: Role.DRIVER,
          target: Target.NEUTRAL,
        }),
        new Actor({
          role: Role.PASSENGER,
          target: Target.START,
        }),
      ],
    });
    expect(wayStepVO.position).toBe(0);
    expect(wayStepVO.lon).toBe(6.17651);
    expect(wayStepVO.lat).toBe(48.689445);
    expect(wayStepVO.actors).toHaveLength(2);
  });
  it('should throw an exception if actors is empty', () => {
    try {
      new WayStep({
        lat: 48.689445,
        lon: 6.17651,
        position: 0,
        actors: [],
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
  it('should throw an exception if actors contains more than one driver', () => {
    try {
      new WayStep({
        lat: 48.689445,
        lon: 6.17651,
        position: 0,
        actors: [
          new Actor({
            role: Role.DRIVER,
            target: Target.NEUTRAL,
          }),
          new Actor({
            role: Role.DRIVER,
            target: Target.START,
          }),
        ],
      });
    } catch (e: any) {
      expect(e).toBeInstanceOf(ArgumentOutOfRangeException);
    }
  });
});
