import { ArgumentInvalidException, ValueObject } from '@mobicoop/ddd-library';
import { PointProps } from './point.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface WaypointProps extends PointProps {
  position: number;
}

export class Waypoint extends ValueObject<WaypointProps> {
  get position(): number {
    return this.props.position;
  }

  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  protected validate(props: WaypointProps): void {
    if (props.position < 0)
      throw new ArgumentInvalidException(
        'position must be greater than or equal to 0',
      );
  }
}
